import logger from 'logger/logger'
import { calcNetworkFee, checkPayResultSign, checkSha1Sign } from 'service/alchemy.svr'
import { PayRecord, PayStatus } from 'modules/PayRecord'
import { TransferQueue } from 'queue/transfer.queue'
import { TransferRecord } from 'modules/TransferRecord'
import { reportPayResult } from 'service/game.svr'
import { PriceSvr } from 'service/price.svr'
import { OrderCacheSvr } from 'service/ordercache.svr'
import { BaseController, role, ROLE_ANON, router, ZError } from 'zutils'
import { fromWei, toBigWei } from 'zutils/utils/bn.util'

let errorRes = function (msg: string) {
  logger.info(`error res: ${msg}`)
  return {
    direct: 1,
    data: null,
    success: false,
    returnCode: '9999',
    returnMsg: msg,
  }
}
/**
 * for Alchemy call,
 */
class AlchemyOutController extends BaseController {
  // 购买回调
  @role(ROLE_ANON)
  @router('post /pay/out/alchemy/result')
  async buyresult(req, res) {
    logger.db('alchemy_result', req)
    let { orderNo, status, crypto, network, merchantOrderNo } = req.params
    logger.info(`alchemy callback: ${orderNo}, ${status}, ${crypto}, ${network}, ${merchantOrderNo}`)
    if (!merchantOrderNo) {
      logger.info(`alchemy callback merchantOrderNo not found, ordreNo: ${orderNo}`)
      throw new ZError(11, 'alchemy callback merchantOrderNo not found')
    }

    let record = await PayRecord.findById(merchantOrderNo)
    if (!record) {
      logger.info(`alchemy callback record not found, ordreNo: ${orderNo}`)
      throw new ZError(12, 'alchemy callback record not found')
    }
    if (record.status !== PayStatus.PENDING && record.status !== PayStatus.PAYED) {
      logger.info(`alchemy callback record status error, ordreNo: ${orderNo}`)
      throw new ZError(13, 'alchemy callback record status error')
    }
    if (!checkPayResultSign(req.params)) {
      logger.info(`alchemy callback sign error: ${orderNo}`)
      throw new ZError(14, `alchemy callback sign error: orderNo: ${orderNo}`)
    }
    if (status === 'FINISHED') {
      let transferRecord = await TransferRecord.findByRecordId(record.id)
      if (transferRecord) {
        transferRecord.status = 9
        await transferRecord.save()
      }
    }
    record.outOrderId = orderNo
    record.network = network
    record.crypto = crypto
    record.outDatas.push(req.params)
    if (req.params.payTime) {
      record.payTime = req.params.payTime
    }
    if (status === 'PAY_SUCCESS') {
      record.status = PayStatus.PAYED
    } else if (status === 'FINISHED') {
      record.status = PayStatus.SUCCESS
    } else {
      record.status = PayStatus.FAIL
    }
    await record.save()
    if (status === 'FINISHED') {
      setImmediate(async () => {
        try {
          await reportPayResult(record)
          record.reportStatus = 1
          await record.save()
        } catch (err) {
          logger.info(`report pay result:: ${record.outOrderId} error: ${err.message}`)
          record.reportStatus = 99
          await record.save()
        }
      })
    }
    logger.info(`alchemy callback success, status: ${status} orderNo: ${orderNo} `)
    return {}
  }

  /**
   * 向商户查询币价
   * TODO:: calc networkFee
   */
  @role(ROLE_ANON)
  @router('get /pay/out/alchemy/queryprice')
  async queryToken(req, res) {
    const { crypto } = req.params
    logger.info(`alchemy query price: ${crypto}`)
    let { appId, appid, timestamp, sign } = req.headers
    logger.info(`alchemy query price headers: ${appid}, ${timestamp}, ${sign}`)
    if (!crypto) {
      return errorRes('params mismatch')
    }
    appId = appId || appid
    if (!appId || !timestamp || !sign) {
      return errorRes('headers mismatch')
    }
    if (!checkSha1Sign(req.headers)) {
      return errorRes('sign error')
    }
    let price = await new PriceSvr().fetchPrice({ crypto: 'ETH', network: 'ARBITRUM', fiat: 'USD' })
    let priceToSend = '0.1'
    if (crypto.toLowerCase() === 'eth' || crypto.toLowerCase() === process.env.CF_ETH_NAME.toLowerCase()) {
      priceToSend = price + ''
    }
    // 20230717 沟通, gas费需要换算成目标货币
    let { networkFee } = await calcNetworkFee(crypto, priceToSend, price)
    let result = {
      direct: 1,
      data: {
        price: priceToSend,
        networkList: [
          {
            network: 'ARBITRUM',
            networkFee: fromWei(networkFee),
          },
        ],
      },
      success: true,
      returnCode: '0000', // false: 9999
      returnMsg: 'in amet',
    }
    return result
  }

  /**
   * 通知商户打币
   */
  @role(ROLE_ANON)
  @router('post /pay/out/alchemy/distribute')
  async distributeToken(req, res) {
    logger.db('alchemy_distribute', req)
    const { orderNo, crypto, network, address, cryptoAmount, cryptoPrice, usdtAmount } = req.params
    logger.info(
      `alchemy distributeToken: orderNo: ${orderNo}, crypto: ${crypto}, network: ${network}, address: ${address}, cryptoAmount: ${cryptoAmount}, cryptoPrice: ${cryptoPrice}, usdtAmount: ${usdtAmount}`,
    )
    let { appId, appid, timestamp, sign } = req.headers
    logger.info(`alchemy distributeToken: appId: ${appId || appid}, timestamp: ${timestamp}, sign: ${sign}`)
    if (!orderNo || !crypto || !network || !address || !cryptoAmount || !cryptoPrice || !usdtAmount) {
      return errorRes('params mismatch')
    }
    appId = appId || appid
    if (!timestamp || !sign) {
      return errorRes('headers mismatch')
    }
    // let signData = { orderNo, crypto, network, address, cryptoAmount, cryptoPrice, usdtAmount }
    if (!checkSha1Sign(req.headers)) {
      return errorRes('sign error')
    }
    if (new OrderCacheSvr().isOrderExist(orderNo)) {
      return errorRes('orderNo already processing')
    }
    new OrderCacheSvr().addOrder(orderNo)
    let record = await PayRecord.findByRecordId(orderNo)
    if (!record) {
      new OrderCacheSvr().removeOrder(orderNo)
      return errorRes('orderNo not found')
    }
    record.outDatas.push(req.params)
    await record.save()
    if (record.crypto != crypto || record.network != network || record.address != address) {
      new OrderCacheSvr().removeOrder(orderNo)
      return errorRes('params mismatch')
    }
    // 简单的比较一下, 如果差距太大, 就不处理
    if (record.cryptoAmountEstimate) {
      let diff = parseFloat(cryptoAmount) / parseFloat(record.cryptoAmountEstimate)
      if (diff > 3) {
        new OrderCacheSvr().removeOrder(orderNo)
        return errorRes('params mismatch, cryptoAmount too big')
      }
    }
    record.cryptoAmount =
      record.network.toLowerCase() === 'agor' && record.crypto.toLowerCase() === 'agor' ? '0.001' : cryptoAmount
    // 20230718 沟通, cryptoAmount包含了gas费, 需要减去后再发送
    let { networkFee } = await calcNetworkFee(crypto, cryptoPrice)
    record.cryptoSend = fromWei(toBigWei(cryptoAmount) - networkFee)
    record.networkFee = fromWei(networkFee)
    record.cryptoPrice = cryptoPrice
    record.usdtAdmount = usdtAmount
    await record.save()
    new TransferQueue().addTask(record)
    let result = {
      direct: 1,
      data: null,
      success: true,
      returnCode: '0000', // false: 9999
      returnMsg: 'in amet',
    }
    new OrderCacheSvr().removeOrder(orderNo)
    return result
  }
}
