import logger from 'logger/logger'
import { GoogleInApp, GooglePayStatus } from 'modules/GoogleInApp'
import { InAppRecord } from 'modules/InAppRecord'
import { IPayResult, reportGooglePurchaseResult } from 'service/game.svr'
import { GooglePaySvr } from 'service/googlepay.svr'
import { BaseController, router, ZError } from 'zutils'

class GooglePayController extends BaseController {
  @router('post /pay/google/verify')
  @router('post /native_pay/google/verify')
  async verifyGooglePay(req, res) {
    logger.db('googlepay_verify', req)
    const { openId } = req.user
    let { list } = req.params
    // check if list is string
    if (typeof list === 'string') {
      try {
        list = JSON.parse(list)
      } catch (err) {
        throw new ZError(10, 'purchase data is empty')
      }
    }
    if (!list || !list.length) {
      throw new ZError(10, 'purchase data is empty')
    }
    logger.info(`verify google purchase::list=${list}`)
    let results: IPayResult[] = []
    for (let sub of list) {
      let infoRes = await new GooglePaySvr().fetchPurchaseData(sub.id, sub.token)
      // logger.log(JSON.stringify(infoRes))
      if (infoRes.status !== 200) {
        throw new ZError(20, 'fetch purchase data failed')
      }
      let info = infoRes.data
      await InAppRecord.addRecord(openId, info, 'google')
      let status = GooglePayStatus.PENDING
      if (info.purchaseState === 0 && info.consumptionState === 0) {
        status = GooglePayStatus.PURCHASED
      }
      if (status === GooglePayStatus.PURCHASED) {
        try {
          let record = await GoogleInApp.insertOrUpdate(
            { outOrderId: info.orderId },
            {
              account: openId, 
              purchaseToken: sub.token,
              productId: sub.id,
              gameOrderId: info.obfuscatedExternalAccountId,
              country: info.regionCode,
              status,
              kind: info.kind,
              purchaseTime: info.purchaseTimeMillis,
              data: info,
              $inc: { version: 1 },
            },
          )
          await new GooglePaySvr().consumePurchase(sub.id, sub.token)
          record.status = GooglePayStatus.SUCCESS
          record.consumeTime = Date.now()
          await record.save()
          if (!record.reportStatus) {
            results.push({
              productId: sub.id,
              gameOrderId: record.gameOrderId,
              orderId: record.outOrderId,
              status: record.status,
            })
          }
        } catch (err) {
          logger.info(`consume purchase:: productId: ${sub.id} purchaseToken: ${sub.token} failed`, err.message || err)
        }
      }
    }
    //TODO:: 通知游戏服
    if (results.length) {
      setImmediate(async () => {
        try {
          await reportGooglePurchaseResult(results)
        } catch (err) {
          logger.error('report google voided purchases failed', err.message || err)
        }
      })
    }

    return results
  }
}
