import { getModelForClass, index, modelOptions, pre, prop, ReturnModelType } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'

import { customAlphabet } from 'nanoid'

const nanoid = customAlphabet('1234567890', 6)

export const DEFAULT_CODE = '000000'
export const DEFAULT_EXPIRE_TIME = 5 * 60 * 1000

export enum CodeType {
  REGIST = 1, // 注册
  RESET = 2, // 重置密码
  VERIFY = 3, // 验证邮箱
}

export enum CodeStatus {
  PENDING = 1,
  SUCCESS = 2,
  FAIL = 3,
  EXPIRED = 4,
}

/**
 * 邮件验证码发送记录
 */
@dbconn()
@index({ email: 1, type: 1 }, { unique: true, partialFilterExpression: { status: 1 } })
@index({ code: 1 }, { partialFilterExpression: { status: 1 } })
@modelOptions({
  schemaOptions: { collection: 'code_send_record', timestamps: true },
})
@pre<CodeRecordClass>('save', async function () {
  if (this.code === DEFAULT_CODE) {
    let exists = false
    while (!exists) {
      const code = nanoid()
      const record = await CodeRecord.findByCode(code, this.type)
      if (!record) {
        exists = true
        this.code = code
      }
    }
  }
})
class CodeRecordClass extends BaseModule {
  @prop({ required: true })
  public email!: string

  @prop()
  public account?: string

  @prop({ required: true })
  public code!: string

  @prop({ default: Date.now() + DEFAULT_EXPIRE_TIME })
  public expiredAt?: number

  @prop({ required: true, default: CodeType.REGIST })
  public type: CodeType

  @prop({ required: true, default: CodeStatus.PENDING })
  public status: CodeStatus

  @prop({ default: false })
  public mailSend: boolean

  @prop()
  public emailId?: string

  public static async findByCode(this: ReturnModelType<typeof CodeRecordClass>, code: string, type: CodeType) {
    return this.findOne({ code, type, status: CodeStatus.PENDING }).exec()
  }

  public static async findByEmail(this: ReturnModelType<typeof CodeRecordClass>, email: string, type: CodeType) {
    return this.findOne({ email, type, status: CodeStatus.PENDING }).exec()
  }
}

export const CodeRecord = getModelForClass(CodeRecordClass, { existingConnection: CodeRecordClass.db })
