import { getModelForClass, index, modelOptions, mongoose, prop, ReturnModelType, Severity } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'
import logger from 'logger/logger'
import { IPayResult, reportGooglePurchaseResult } from 'service/game.svr'

export enum GooglePayStatus {
  PENDING = 0, // 默认状态, 未支付
  TRANSFERING = 1, //只有购买链上资产才会有该状态
  TRANSFERED = 2, //只有购买链上资产才会有该状态
  PURCHASED = 3, //用户已支付, 但未确认和消费
  SUCCESS = 9,
  VOIDED = 96, // 作废, 取消, 退款
  USER_CANCEL = 97, // 用户取消
  TRANSFER_FAIL = 98, // 转账错误
  FAIL = 99, // 交易失败
}

@dbconn('pay')
@index({ gameOrderId: 1 }, { unique: true, partialFilterExpression: { gameOrderId: { $exists: true } } })
@index({ outOrderId: 1 }, { unique: true })
@index({ account: 1 }, { unique: false })
@modelOptions({
  schemaOptions: { collection: 'google_inapp_record_202410', timestamps: true },
  options: { allowMixed: Severity.ALLOW },
})
export class GoogleInAppClass extends BaseModule {
  // 用户账号
  @prop({ required: true })
  public account: string  // 这里只表示谁来消耗这个订单, 不代表购买者
  // 渠道返回的订单id
  @prop()
  public outOrderId: string
  // 国家, 这里取自订单信息中返回的国家代码
  @prop()
  public country?: string
  // 游戏服传入的订单id,
  // 由于有非游戏内购买的可能, 所以该字段可能不存在
  @prop()
  public gameOrderId?: string
  // 交易状态
  @prop({ required: true, default: GooglePayStatus.PENDING })
  public status: GooglePayStatus
  // 渠道返回的原始资料
  @prop({ type: mongoose.Schema.Types.Mixed })
  public outData: any

  // 对应google play console中的 productid
  @prop({ required: true })
  public productId: string
  @prop()
  public purchaseToken: string

  @prop({ required: true, default: 1 })
  public amount: number
  // 表示 androidpublisherservice 中的 inappPurchase 对象
  @prop()
  public kind: string

  // 支付时间, 取自渠道订单信息中的purchaseTimeMillis
  @prop()
  public purchaseTime: number
  @prop()
  public gameAccountId?: string
  // 消费时间
  @prop()
  public consumeTime?: number

  // voided 相关信息
  @prop()
  public voidedTime?: number
  // 发起作废购买交易的发起者（可能的值：0）。用户 1. 开发者 2. Google
  @prop()
  public voidedSource?: number
  // 购买作废的原因可能为：0。其他 1. 反悔 2。未收到： 3. 有缺陷 4. Accidental_purchase 5. 欺诈 6. Friendly_fraud 7. 退款
  @prop()
  public voidedReason?: number

  @prop({ default: 0 })
  public version: number

  @prop({ default: 0 })
  public reportStatus: number

  public static async findByRecordId(this: ReturnModelType<typeof GoogleInAppClass>, outOrderId: string) {
    return this.findOne({ outOrderId }).exec()
  }

  public static async parseVoidedRecords(this: ReturnModelType<typeof GoogleInAppClass>, voidedPurchases: any) {
    if (!voidedPurchases || !voidedPurchases.length) {
      return
    }
    let results: IPayResult[] = []
    for (let sub of voidedPurchases) {
      let { orderId, voidedTimeMillis, voidedSource, voidedReason } = sub
      // orderId = 'GPA.3355-1172-9416-16839'
      const record = await GoogleInApp.findOneAndUpdate(
        { outOrderId: orderId, status: GooglePayStatus.SUCCESS },
        { $set: { voidedTime: voidedTimeMillis, voidedSource, voidedReason, status: GooglePayStatus.VOIDED } },
        { returnDocument: 'after' },
      )
      if (record) {
        results.push({
          productId: record.productId,
          gameOrderId: record.gameOrderId,
          orderId: record.outOrderId,
          status: record.status,
        })
      }
    }

    if (results.length) {
      try {
        await reportGooglePurchaseResult(results)
      } catch (err) {
        logger.error('report google voided purchases failed', err.message || err)
      }
    }
  }
}

export const GoogleInApp = getModelForClass(GoogleInAppClass, { existingConnection: GoogleInAppClass.db })
