import { getModelForClass, index, modelOptions, mongoose, prop, ReturnModelType, Severity } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'

export enum PayPlatEnum {
  ALCHEMY = 1,
}

export enum PayType {
  BUY = 1,
  SELL = 2,
}

export enum PayStatus {
  PENDING = 0, // 默认状态, 未支付
  PAYED = 3, // 已支付
  SUCCESS = 9,
  PAY_FAIL = 97, // 用户支付失败, 国库模式
  FAIL = 99, // 交易失败
}

@dbconn('pay')
@index({ outOrderId: 1 }, { unique: true, partialFilterExpression: { outOrderId: { $exists: true } } })
@index({ account: 1, crypto: 1, status: 1, createdAt: 1 }, { unique: false })
@modelOptions({
  schemaOptions: { collection: 'pay_record', timestamps: true },
  options: { allowMixed: Severity.ALLOW },
})
export class PayRecordClass extends BaseModule {
  // 支付渠道
  @prop({ enum: PayPlatEnum, default: PayPlatEnum.ALCHEMY })
  public channel!: PayPlatEnum
  // 交易类型
  @prop({ required: true, default: PayType.BUY })
  public type: PayType
  // 用户账号
  @prop({ required: true })
  public account: string
  // 用户钱包地址
  @prop()
  public address: string
  // 购买的网络名, 根据alchemy的定义
  @prop()
  public network?: string
  // 购买的发币名称
  @prop()
  public crypto?: string

  @prop()
  public env?: string

  // 法币
  @prop()
  public fiat?: string
  // 法币数量
  @prop()
  public fiatAmount?: string
  // 加密货币数量,
  @prop()
  public cryptoAmount?: string
  // 实际转账金额
  @prop()
  public cryptoSend?: string

  @prop()
  public networkFee?: string

  // 开始创建记录时, 估算的可获取加密货币数量
  @prop()
  public cryptoAmountEstimate?: string

  // 加密货币价格
  @prop()
  public cryptoPrice?: string
  // 该笔交易渠道会给我们多少usdt
  @prop()
  public usdtAdmount?: string
  // 国家
  @prop()
  public country?: string
  @prop()
  public payWayCode?: string
  // 交易状态
  @prop({ required: true, default: PayStatus.PENDING })
  public status: PayStatus

  // 渠道返回的原始资料
  @prop({ type: mongoose.Schema.Types.Mixed })
  public outData: any

  @prop({ type: () => [mongoose.Schema.Types.Mixed] })
  public outDatas: any[]

  // 支付时间, 由alchemy提供
  @prop()
  public payTime: string

  // 渠道返回的订单id
  @prop()
  public outOrderId: string
  // 交易的txHash
  @prop()
  public txHash?: string

  @prop()
  public gameAccountId: string
  @prop()
  public gameOrderId: string
  /**
   * 国库模式转账状态
   * 0: 未转账
   * 1: 提交转账成功
   * 2: 转账成功
   * 99: 转账失败
   */
  @prop({ default: 0 })
  public tranStatus: number

  /**
   * 国库模式通知alchemy次数
   */
  @prop({ default: 0 })
  public notofyErrCount: number

  /**
   * 上报游戏服状态
   * 0: 未上报
   * 1: 上报成功
   * 99: 上报失败
   */
  @prop({ default: 0 })
  public reportStatus: number
  /**
   * 客户端版本号
   */
  @prop({ default: 0 })
  public versionCode: number

  public static async findByRecordId(this: ReturnModelType<typeof PayRecordClass>, outOrderId: string) {
    return this.findOne({ outOrderId }).exec()
  }
}

export const PayRecord = getModelForClass(PayRecordClass, { existingConnection: PayRecordClass.db })
