import { AppStoreServerAPI } from 'apple/AppStoreServerAPI'
import { decodeNotificationPayload, decodeTransaction, decodeTransactions } from 'apple/Decoding'
import {
  DecodedNotificationPayload,
  Environment,
  OrderLookupStatus,
  ProductTypeParameter,
  SortParameter,
} from 'apple/Models'

import logger from 'logger/logger'
import { singleton } from 'zutils'

const env = process.env.NODE_ENV || 'development'
const sandbox = env === 'development' ? true : false

const KEY = `-----BEGIN PRIVATE KEY-----
${process.env.IOS_PRIVATE_KEY}
-----END PRIVATE KEY-----`

const KEY_ID = process.env.IOS_KEY_ID
const ISSUER_ID = process.env.IOS_ISSUER_ID
const APP_BUNDLE_ID = process.env.IOS_APP_BUNDLE_ID
const environment = sandbox ? Environment.Sandbox : Environment.Production

const apiProd = new AppStoreServerAPI(KEY, KEY_ID, ISSUER_ID, APP_BUNDLE_ID, Environment.Production)
const apiSandbox = new AppStoreServerAPI(KEY, KEY_ID, ISSUER_ID, APP_BUNDLE_ID, Environment.Sandbox)

@singleton
export class IosPaySvr {
  public async iosPayVerify(orderId: string) {
    logger.info('iosPayVerify: ', orderId)
    let response
    try {
      response = await apiProd.lookupOrder(orderId)
    } catch (err) {
      logger.info('iosPayVerify err: ', err)
      response = await apiSandbox.lookupOrder(orderId)
    }

    if (response.status === OrderLookupStatus.Valid) {
      const transactions = await decodeTransactions(response.signedTransactions)
      logger.info('transactions: ', transactions)
    }
  }

  public async getTransactionHistory(originalTransactionId: string, rversion: string = '') {
    let reqData = {
      productType: ProductTypeParameter.Consumable,
      sort: SortParameter.Descending,
    }
    if (rversion) {
      reqData['revision'] = rversion
    }
    let response
    try {
      response = await apiProd.getTransactionHistory(originalTransactionId, reqData)
    } catch (err) {
      logger.info(`transaction with id ${originalTransactionId} not found in production, try sandbox`)
      try {
        response = await apiSandbox.getTransactionHistory(originalTransactionId, reqData)
      } catch (errSandbox) {
        logger.info(`transaction with id ${originalTransactionId} not found in sandbox`)
      }
    }
    let results: any = []
    // Decoding not only reveals the contents of the transactions but also verifies that they were signed by Apple.
    const transactions = await decodeTransactions(response.signedTransactions)
    for (let transaction of transactions) {
      // Do something with your transactions...
      results.push(transaction)
    }
    // The response contains at most 20 entries. You can check to see if there are more.
    if (response.hasMore) {
      let sub = await this.getTransactionHistory(originalTransactionId, response.revision)
      results = results.concat(sub)
    }
    return results
  }

  public async getTransactionInfo(originalTransactionId: string) {
    let response
    try {
      response = await apiProd.getTransactionInfo(originalTransactionId)
    } catch (err) {
      try {
        logger.info(`transaction with id ${originalTransactionId} not found in production, try sandbox`)
        response = await apiSandbox.getTransactionInfo(originalTransactionId)
      } catch (errSandbox) {
        logger.info(`transaction with id ${originalTransactionId} not found in sandbox`)
        throw new Error('getTransactionInfo err')
      }
    }
    if (!response) {
      throw new Error('empty transaction info')
    }
    const transaction = await decodeTransaction(response.signedTransactionInfo)
    return transaction
  }

  public async queryNotificationHistory() {
    // Start and end date are required.
    // The earliest supported start date is June 6th (the start of WWDC 2022).
    const response = await apiProd.getNotificationHistory({
      startDate: 1690175687060, // June 6th 2022
      endDate: new Date().getTime(),
    })
    logger.info('queryNotificationHistory response: ', response)
    for (let notification of response.notificationHistory) {
      let info = await decodeNotificationPayload(notification.signedPayload)
      logger.info('notification: ', info)
    }
    // Check if there are more items.
    if (response.hasMore) {
      // Use history.paginationToken to fetch additional items.
    }
  }

  public async requestTestNotification() {
    let resp = await apiSandbox.requestTestNotification()
    logger.info('requestTestNotification response: ', resp)
    let info = await apiSandbox.getTestNotificationStatus(resp.testNotificationToken)
    let decodeInfo = await decodeNotificationPayload(info.signedPayload)
    logger.info(decodeInfo)
    return decodeInfo
  }

  public async verifyNotification(signedPayload: string) {
    const decodedJWT: DecodedNotificationPayload = await decodeNotificationPayload(signedPayload)
    logger.info('apple notification:: ', JSON.stringify(decodedJWT))

    const appAppleId = decodedJWT.data
      ? decodedJWT.data.appAppleId
      : decodedJWT.summary
      ? decodedJWT.summary.appAppleId
      : null
    const bundleId = decodedJWT.data
      ? decodedJWT.data.bundleId
      : decodedJWT.summary
      ? decodedJWT.summary.bundleId
      : null
    const environment = decodedJWT.data
      ? decodedJWT.data.environment
      : decodedJWT.summary
      ? decodedJWT.summary.environment
      : null
    if (APP_BUNDLE_ID !== bundleId) {
      // throw new VerificationException(VerificationStatus.INVALID_APP_IDENTIFIER)
    }
    // if (env === 'production' && this.appAppleId !== appAppleId) {
    //   // throw new VerificationException(VerificationStatus.INVALID_APPLE_IDENTIFIER)
    // }
    if (env !== environment) {
      // throw new VerificationException(VerificationStatus.INVALID_ENVIRONMENT)
    }
    return decodedJWT
  }
}
