import axios from 'axios'
import { queryPrice } from './alchemy.svr'
import * as schedule from 'node-schedule'
import logger from 'logger/logger'
import { singleton } from 'zutils'

@singleton
export class PriceSvr {
  private priceMap: Map<string, string> = new Map()

  public async queryEthPrice(eth: string, fiat: string = 'USD') {
    const apiKey = process.env.CRYPTOCOMPARE_API_KEY
    const url = `https://min-api.cryptocompare.com/data/price?fsym=${eth}&tsyms=${fiat}&api_key=${apiKey}`
    let priceData = await axios.get(url).then(res => res.data)
    let price = priceData[fiat] ? priceData[fiat] + '' : ''
    return price
  }

  public async fetchPrice(data: any) {
    let { crypto, network, fiat } = data
    let key = `${crypto}_${network}_${fiat}`
    let price = this.priceMap.get(key)
    if (!price) {
      await this.refreshOne(data)
      price = this.priceMap.get(key)
    }
    return price
  }

  private async refreshOne(data) {
    let { crypto, network, fiat } = data
    let key = `${crypto}_${network}_${fiat}`
    try {
      let price = await this.queryEthPrice(crypto, fiat)
      if (price) {
        this.priceMap.set(key, price + '')
      } else {
        let priceData = await queryPrice(data)
        if (priceData.success && priceData.data) {
          this.priceMap.set(key, priceData.data.cryptoPrice + '')
        }
      }
    } catch (e) {
      logger.debug('update eth price with error: ' + e.message || e)
    }
  }

  private async refreshAll() {
    for (let key of this.priceMap.keys()) {
      let [crypto, chain, fiat] = key.split('_')
      let data = {
        crypto,
        network: chain,
        fiat,
      }
      await this.refreshOne(data)
    }
  }

  scheduleAll() {
    const job = schedule.scheduleJob('*/30 * * * * *', async () => {
      this.refreshAll()
    })
  }
}
